@objc(Vidvocr)
import VIDVOCR
class Vidvocr: NSObject, VIDVOCRDelegate {
    let dispatchGroup = DispatchGroup()
    var jsonResponse = ""
    var success = false
    @objc(startOCR:withResolver:withRejecter:)
    func startOCR(_ params: NSDictionary,_ resolve: @escaping RCTPromiseResolveBlock,_ reject: @escaping RCTPromiseRejectBlock) {
        
        IDVCustomOCR.sharedInstance.delegate = self
        
        let baseURL = params.value(forKey: "base_url") as? String ?? ""
        let accessToken = params.value(forKey: "access_token") as? String ?? ""
        let bundle = params.value(forKey: "bundle_key") as? String ?? ""
        let languageString = params.value(forKey: "language") as? String ?? ""
        let hasWatermark  = params.value(forKey: "watermark") as? Bool ?? false
        let hasOCR  = params.value(forKey: "perform_ocr") as? Bool ?? false
        let hasDocVerification  = params.value(forKey: "document_verification") as? Bool ?? false
        let withoutVoiceover  = params.value(forKey: "without_voiceover") as? Bool ?? false
        let showDialogs  = params.value(forKey: "show_dialogs") as? Bool ?? true
        let headers  = params.value(forKey: "headers") as? [String:String] ?? [:]

        DispatchQueue.main.async {[weak self] in
            guard let self = self else {return}
            var builder = OCRBuilder()
            builder = builder.setDataValidation(validate: self.ocrConfSwitch.isOn )
                .setDocumentVerification(verify: self.watermarkSwitch.isOn )
                .setlLanguage(language: self.englishFlagSwitch.isOn ? "en" : "ar")
                .setBundleKey(creds.bundle)
                .setBaseUrl(creds.baseURL)
                .setAccessToken(token ?? "")
                .setReviewData(review: self.reviewDataSwicth.isOn)
                .setCaptureOnlyMode(self.singleCaptureSwitch.isOn)
            builder = builder.setSSLCertificate(certificateData)
            builder = builder.setPrimaryColor(color: .orange)
            builder = builder.setHeaders(headers: ["key": "value", "key2": "value2"])
            let keyWindow = UIApplication.shared.windows.filter {$0.isKeyWindow}.first
            guard var topController = keyWindow?.rootViewController else {return}
            while let presentedViewController = topController.presentedViewController {
                topController = presentedViewController
            }
            builder.start(vc: topController, ocrDelegate: self)
        }
        
        self.dispatchGroup.enter()
        dispatchGroup.notify(queue: DispatchQueue.global()){
            if self.success {
                resolve(self.jsonResponse)
            } else {
                resolve(self.jsonResponse)
            }
        }
    }
    
    func onOCRResult(result: VIDVOCRResponse) {
        var dict = NSMutableDictionary()
        DispatchQueue.main.async {[weak self] in
            guard let self = self else {return}
            self.startBtn.alpha = 1
            self.startBtn.isEnabled = true
        }
        switch result {
        case .success(let data):
            dict = VIDVResultToDict(VIDVResult: data, hasResult: true, errorMessage: nil, errorCode: nil, stage: nil)
            success = true
            dispatchGroup.leave()
        case .builderError(let code, let message):
            dict = VIDVResultToDict(VIDVResult: nil, hasResult: true, errorMessage: message, errorCode: code, stage: nil)
            success = true
            dispatchGroup.leave()
        case .serviceFailure(let code, let message, let data):
            dict = VIDVResultToDict(VIDVResult: data, hasResult: true, errorMessage: message, errorCode: code, stage: nil)
            success = true
            dispatchGroup.leave()
        case .userExit(let step, let data):
            dict = VIDVResultToDict(VIDVResult: data, hasResult: true, errorMessage: nil, errorCode: nil, stage: nil)
            success = true
            dispatchGroup.leave()
        case .capturedImages(image: let image):
//            dict = VIDVResultToDict(VIDVResult: data, hasResult: true, errorMessage: nil, errorCode: nil, stage: nil)
//            success = true
//            dispatchGroup.leave()
            break
        }
        
        do {
            let jsonData = try JSONSerialization.data(withJSONObject: dict, options: .prettyPrinted)
            let json = String(data: jsonData, encoding: .utf8)
            self.jsonResponse = json ?? ""
            //                print(jsonResponse)
            dispatchGroup.leave()
        } catch let error as NSError {
            print("Failed to load: \(error.localizedDescription)")
        }
    }
    
    func VIDVResultToDict(VIDVResult: VIDVOCRResult?, hasResult: Bool, errorMessage: String?, errorCode: Int?, stage: String?) -> NSMutableDictionary {
        
        var dict = NSMutableDictionary()
        var nameValuePairs = NSMutableDictionary()
        var valifyData = NSMutableDictionary()
        
        dict["nameValuePairs"] = nameValuePairs
        
        
        if errorCode != nil {
            nameValuePairs["errorCode"] = errorCode
        }
        if errorMessage != nil {
            nameValuePairs["errorMessage"] = errorMessage
        }
        if stage != nil {
            nameValuePairs["stage"] = stage
        }
        
        if hasResult {
            do {
                let jsonEncoder = JSONEncoder()
                let jsonData = try jsonEncoder.encode(VIDVResult)
                let jsonVIDVResult = String(data: jsonData, encoding: .utf8)
                nameValuePairs["valifyData"] = jsonVIDVResult
            } catch {
                print("Failed to encode VIDV Result: \(error.localizedDescription)")
            }
        }
        
        return dict
    }
}
